/*globals require, window*/

require([
    'testsuite/qunit',
    'testsuite/sinon',
    'testsuite/qunit-fixture!de_epages/openai/ui/ai-content-test',
    'jquery',
    'ep',
    'de_epages/openai/ui/ai-content',
    "$dict!de_epages/openai/ui/ai-content",

    'ep/ui/validate',
    'jquery/i18n',
    'jquery/metaparse'
], function (QUnit, sinon, fixture, $, ep) {
    'use strict';

    /*
     * @dictionary       de_epages.product.dictionary
     *
     * @translation      {IsGross}
     *                   {IsNet}
     *                   {WithoutShipping}
     */

    /*
     * @dictionary      ep.dict
     */

    var methods = [
            '_create',
            '_setOption',
            '_buildOptimizeDialog',
            '_renderOptimizeDialog',
            '_openAiRequest',
            '_confirm',
            '_confirmRequest',
            '_onAfterGenerate',
            '_setCopyState',
            '_toggleOptimizeDialog',
            '_copyToClipboard',
            '_handleExecCommand',
            '_lineParser',
            'open',
            'close',
            'destroy'
        ],

        sandbox = sinon.sandbox.create(),

        exampleDescription = "Don't be a pickle, be a pineapple! Stand tall, wear a crown, and be sweet on the inside.",
        scribeDescription = "(Scribe content) - Don't be a pickle, be a pineapple! Stand tall, wear a crown, and be sweet on the inside.",
        // mocked scribe instance
        scribe = {
            content: scribeDescription,
            element: {
                text: function() {
                    return scribe.content;
                }
            },
            scribe: {
                setHTML: function (content) {
                    // create code fragment to test if paragraph node tag is found at least one time
                    var codeFragment = $('<p />').html(content);

                    QUnit.ok(
                        // test if paragraph node is found at least one time
                        codeFragment.find(paragraphNode).length,
                        'Formatted response uses "' + paragraphNode + '" as paragraph node tag.'
                    );

                    scribe.content = content;
                }
            }
        },

        remainingTokens = 1000,
        paragraphNode = "div",

        // simple tokenizer mock
        tokenizer = function(message) {
            return message.length;
        },

        lineParser = function(text) {
            return text
                .replace(/\n\n+/g, "\n\n")
                .split("\n")
                .map(
                    function(line) {
                        return "<" + paragraphNode + ">" + line + "</" + paragraphNode + ">";
                })
                .join("");
        };

    /* START - fake ajax request */
    // ep.ajax = function(o) {
    ep.ajax = function(o) {
        var result = $.Deferred();

        if (o.data && o.data.ChangeAction) {
            if (o.data.ChangeAction === "JSONOptimizeAiContent") {

                window.setTimeout(function() {
                    var message = (function(slogans){
                            return slogans[Math.floor((Math.random() * 10000) % slogans.length)];
                        })([
                            "We put the 'laughter' in 'halftime slaughter'!",
                            "No pain, no gain, no brain—must be a sport!",
                            "We may be slow, but at least we're still faster than couch potatoes!",
                            "With us, even the referees score more points!",
                            "Sorry for warming up, we thought this was a pizza place!",
                            "Our opponents are like parking tickets—they always get stuck!",
                            "We may be a team, but we know that 'I' comes before 'U' in individual efforts!",
                            "Running bases and eating pizza—our two favorite ways of getting greasy!",
                            "We're all about 'sports'anity instead of 'sanity'!",
                            "Our team motto: 'Don't chase the ball, chase the snacks!'",
                            "We may not have skills, but we've mastered the art of faking it!",
                            "Sports is like bacon—makes everything better (except vegetarians)!",
                            "Our practice sessions are like yoga—we mostly stretch the truth!",
                            "We may not be fast, but at least we make the spectators laugh!",
                            "We're the official team of 'Finding Nemo'—we're always 'tanked'!",
                            "We may not win, but at least we're a pro at over-celebrating!",
                            "Our sports strategy: 'Distract and confetti!'",
                            "Our team is the reason they invented instant replay—pure entertainment!",
                            "Sports is our cardio, and tacos are our motivation!",
                            "We're so bad at sports that even the ball tries to escape!",
                            "Cheer for us if you believe in miracles, unicorns, and sports!",
                            // test response string with more than one paragraph
                            "... and here is! ... \n\nthe new and super and sexy and AI optimized content *jingle ..."
                        ]);

                    result.resolve({
                        success: true,
                        remaining_tokens: remainingTokens - tokenizer(message),
                        content: message
                    });
                }, 250);
            } else if (o.data.ChangeAction === "JSONSaveSeenAIDialog") {
                QUnit.equal(
                    o.data.IsAIDialogConfirmed,
                    1,
                    'Confirmation flag is set.'
                );

                result.resolve({
                    success: true,
                    confirmed: true
                });
            } else {
                result.reject();
            }
        }

        return result;
    };
    /* END - fake ajax request */

    QUnit.module('de_epages/openai/ui/ai-content', {
        teardown: function() {
            fixture.destroy();
            sandbox.restore();
        }
    });

    QUnit.fixtureVar = fixture;

    QUnit.test('de_epages/openai/ui/ai-content - check existence of ai-content methods', methods.length, function() {
        $.each(methods, function() {
            QUnit.ok(
                $.isFunction($.ui.openaiAiContent.prototype[this]),
                this + ' is a method of the ai-content widget'
            );
        });
    });

    QUnit.asyncTest('de_epages/openai/ui/ai-content - widget basic tests', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        window.setTimeout(function() {
            // metaparse form elements
            $('#qunit-fixture .ep-js').metaparse();

            var aiDescriptionButton = $('.ep-product-ai-description-btn'),

                widgetInstance;

            // prepare spies
            sandbox.spy($.ui.openaiAiContent.prototype, '_create');
            sandbox.spy($.ui.openaiAiContent.prototype, '_buildOptimizeDialog');
            sandbox.spy($.ui.openaiAiContent.prototype, '_renderOptimizeDialog');

            aiDescriptionButton.openaiAiContent({
                // Variant 1: Configuration using a callback function
                onApply: function(content) {
                  console.log('Write new content to console with a callback function:', content);
                },
                description: "#Description",
                // Variant 2: Configuration using a scribe instance
                // scribe: scribeEditor,

                // Configuration for both variants
                changeActionAI: "JSONOptimizeAiContent",
                changeActionConfirm: "JSONSaveSeenAIDialog",
                isConfirmed: false
            });

            // wait for fixture and widget
            window.setTimeout(function() {
                widgetInstance = aiDescriptionButton.data('uiOpenaiAiContent');

                QUnit.ok(
                    widgetInstance !== undefined,
                    'Widget exists'
                );
                // check if spy in create method was called
                QUnit.ok(
                    sandbox.fakes[0].callCount === 1,
                    '"_create" method was called ' + sandbox.fakes[0].callCount + ' times.'
                );
                QUnit.ok(
                    sandbox.fakes[2].callCount === 0,
                    '"_renderOptimizeDialog" method was called ' + sandbox.fakes[2].callCount + ' times.'
                );
                // trigger click to open dialog
                aiDescriptionButton.trigger('click');

                window.setTimeout(function() {
                    // check if spy in create method was called
                    QUnit.ok(
                        sandbox.fakes[1].callCount === 1,
                        '"_create" method was called ' + sandbox.fakes[1].callCount + ' times.'
                    );
                    QUnit.ok(
                        sandbox.fakes[2].callCount > 0,
                        '"_renderOptimizeDialog" method was called ' + sandbox.fakes[2].callCount + ' times.'
                    );

                    // destroy widget
                    aiDescriptionButton.openaiAiContent('destroy');

                    window.setTimeout(function() {
                        QUnit.ok(
                            aiDescriptionButton.data('uiOpenaiAiContent') === undefined,
                            'Widget "uiOpenaiAiContent" was destroyed'
                        );
                        widgetInstance.optimizeDialog && QUnit.ok(
                            widgetInstance.optimizeDialog.data('uiUiDialog') === undefined,
                            'Widget "optimizeDialog" was destroyed'
                        );
                        widgetInstance.confirmDialog && QUnit.ok(
                            widgetInstance.confirmDialog.data('uiUiDialog') === undefined,
                            'Widget "confirmDialog" was destroyed'
                        );

                        QUnit.equal(
                            !!(
                                $._data(widgetInstance.tmplElements.descrSource[0], "events") &&
                                $._data(widgetInstance.tmplElements.descrSource[0], "events").input
                            ),
                            false,
                            "input event was successfully removed from source description element by *destroy*."
                        );
                        QUnit.equal(
                            !!(
                                $._data(widgetInstance.tmplElements.descrTarget[0], "events") &&
                                $._data(widgetInstance.tmplElements.descrTarget[0], "events").input
                            ),
                            false,
                            "input event was successfully removed from target description element by *destroy*."
                        );
                        QUnit.equal(
                            !!(
                                $._data(widgetInstance.tmplElements.optimizeButton[0], "events") &&
                                $._data(widgetInstance.tmplElements.optimizeButton[0], "events").click
                            ),
                            false,
                            "input event was successfully removed from the optimize button by *destroy*."
                        );
                        QUnit.equal(
                            !!(
                                $._data(widgetInstance.tmplElements.copyButton[0], "events") &&
                                $._data(widgetInstance.tmplElements.copyButton[0], "events").click
                            ),
                            false,
                            "input event was successfully removed from the copy button by *destroy*."
                        );

                        QUnit.start();
                    }, 50);
                }, 100);

            }, 50);
        }, 200);
    });

    QUnit.asyncTest('de_epages/openai/ui/ai-content - widget ui tests', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        window.setTimeout(function() {
            // metaparse form elements
            $('#qunit-fixture .ep-js').metaparse();

            var aiDescriptionButton = $('.ep-product-ai-description-btn'),

                confirmBackButton,
                confirmReplaceButton,
                replaceDescription,
                widgetInstance;

            // prepare spies
            sandbox.spy($.ui.openaiAiContent.prototype, '_create');
            sandbox.spy($.ui.openaiAiContent.prototype, '_buildOptimizeDialog');

            aiDescriptionButton.openaiAiContent({
                // Variant 1: Configuration using a callback function
                onApply: function(content) {
                    console.log('Write new content to console with a callback function:', content);
                    QUnit.equal(
                        content,
                        replaceDescription,
                        'New content is "' + replaceDescription + '".'
                    );
                },
                description: exampleDescription,
                // Variant 2: Configuration using a scribe instance
                // scribe: scribeEditor,

                // Configuration for both variants
                changeActionAI: "JSONOptimizeAiContent",
                changeActionConfirm: "JSONSaveSeenAIDialog",
                isConfirmed: false,
                remainingTokens: remainingTokens,
                paragraphNode: paragraphNode
            });

            // wait for fixture and widget
            window.setTimeout(function() {
                widgetInstance = aiDescriptionButton.data('uiOpenaiAiContent');

                QUnit.ok(
                    widgetInstance !== undefined,
                    'Widget exists'
                );
                // check if spy in create method was called
                QUnit.ok(
                    sandbox.fakes[0].callCount === 1,
                    '"_create" method was called ' + sandbox.fakes[0].callCount + ' times.'
                );

                // trigger click to open dialog
                aiDescriptionButton.trigger('click');

                window.setTimeout(function() {
                    // check if spy in create method was called
                    QUnit.ok(
                        sandbox.fakes[1].callCount === 1,
                        '"buildOptimizeDialogSpy" method was called ' + sandbox.fakes[1].callCount + ' times.'
                    );

                    // check dialog instance
                    QUnit.ok(
                        widgetInstance.optimizeDialog.data('uiDialog') !== undefined,
                        'Optimize dialog exists'
                    );
                    QUnit.ok(
                        widgetInstance.optimizeDialog.closest('.' + widgetInstance.optimizeDialogClassName).is(':visible'),
                        'Optimization dialog is visible.'
                    );

                    // check text area for optimized description and buttons initially
                    QUnit.equal(
                        widgetInstance.tmplElements.descrSource.val(),
                        exampleDescription,
                        'Content in text area for existing description is "' + exampleDescription + '".'
                    );
                    QUnit.ok(
                        true,
                        '=== Check text area for optimized description and buttons initiallyI ==='
                    );
                    QUnit.equal(
                        widgetInstance.tmplElements.descrTarget.val().length,
                        0,
                        'Text area for optimized description is empty.'
                    );
                    QUnit.ok(
                        widgetInstance.tmplElements.descrTarget[0].disabled,
                        'Text area for optimized description is disabled.'
                    );
                    QUnit.equal(
                        widgetInstance.optimizeDialog.data('uiUiDialog').uiButtonSet.find('.' + widgetInstance.confirmButtonClass).prop('disabled'),
                        true,
                        'Apply button is disabled.'
                    );
                    QUnit.equal(
                        widgetInstance.tmplElements.remainingTokens.text(),
                        $.i18n.formatNumber(remainingTokens, 'n0'),
                        'Initial number of "' + remainingTokens + ' remaining tokes" is rendered correctly.'
                    );
                    QUnit.equal(
                        widgetInstance.tmplElements.copyButton.prop('disabled'),
                        true,
                        'Copy button is disabled.'
                    );

                    // trigger optimize button
                    widgetInstance.tmplElements.optimizeButton.trigger('click');

                    window.setTimeout(function() {
                        // check text area for optimized description and buttons after requesting the OpenAI API
                        QUnit.ok(
                            true,
                            '=== Check text area for optimized description and buttons after requesting the OpenAI API ==='
                        );
                        QUnit.ok(
                            widgetInstance.tmplElements.descrTarget.val().length > 0,
                            'Text area for optimized description is filled.'
                        );
                        QUnit.ok(
                            !widgetInstance.tmplElements.descrTarget[0].disabled,
                            'Text area for optimized description is enabled.'
                        );

                        // calculate expected updated number of remaining tokens
                        remainingTokens = remainingTokens - widgetInstance.tmplElements.descrTarget[0].value.length;
                        QUnit.equal(
                            widgetInstance.tmplElements.remainingTokens.text(),
                            $.i18n.formatNumber(remainingTokens, 'n0'),
                            'Updated number of "' + remainingTokens + ' remaining tokes" was calculated and rendered correctly.'
                        );
                        QUnit.equal(
                            widgetInstance.optimizeDialog.data('uiUiDialog').uiButtonSet.find('.' + widgetInstance.confirmButtonClass).prop('disabled'),
                            false,
                            'Apply button is enabled.'
                        );
                        QUnit.equal(
                            widgetInstance.tmplElements.copyButton.prop('disabled'),
                            false,
                            'Copy button is enabled.'
                        );

                        // test apply button
                        widgetInstance.buttons.apply.trigger('click');

                        window.setTimeout(function() {
                            // check confirmation dialog
                            QUnit.ok(
                                true,
                                '=== Check confirmation dialog ==='
                            );
                            QUnit.ok(
                                !widgetInstance.optimizeDialog.closest('.' + widgetInstance.optimizeDialogClassName).is(':visible'),
                                'Optimization dialog is hidden.'
                            );
                            QUnit.ok(
                                widgetInstance.confirmDialog.closest('.' + widgetInstance.confirmDialogClassName).is(':visible'),
                                'Confirmation dialog is visible.'
                            );

                            confirmBackButton = widgetInstance.confirmDialog.data('uiUiDialog').uiButtonSet.find('.ep-confirm-back');
                            confirmReplaceButton = widgetInstance.confirmDialog.data('uiUiDialog').uiButtonSet.find('.' + widgetInstance.confirmButtonClass);

                            confirmBackButton.trigger('click');

                            window.setTimeout(function() {
                                QUnit.ok(
                                    widgetInstance.optimizeDialog.closest('.' + widgetInstance.optimizeDialogClassName).is(':visible'),
                                    'Optimization dialog is visible.'
                                );
                                QUnit.ok(
                                    !widgetInstance.confirmDialog.closest('.' + widgetInstance.optimizeDialogClassName).is(':visible'),
                                    'Confirmation dialog is hidden.'
                                );


                                // check replace button AND confirmation flag
                                QUnit.ok(
                                    true,
                                    '=== Check replace button AND confirmation flag ==='
                                );

                                replaceDescription = widgetInstance.tmplElements.descrTarget[0].value;

                                widgetInstance.buttons.apply.trigger('click');

                                window.setTimeout(function() {

                                    // check confirmation flag
                                    widgetInstance.tmplElements["confirmInput"].prop('checked', true);
                                    confirmReplaceButton.trigger('click');

                                    window.setTimeout(function() {
                                        QUnit.ok(
                                            true,
                                            '=== Test scribe functionality ==='
                                        );

                                        aiDescriptionButton.openaiAiContent('option', 'scribe', scribe);

                                        QUnit.ok(
                                            widgetInstance.options.scribe == scribe,
                                            'Scribe editor (WYSIWYG) has been set correctly as an option.'
                                        );

                                        // trigger click to open dialog
                                        aiDescriptionButton.trigger('click');

                                        window.setTimeout(function() {
                                            // check text area for optimized description and buttons after setting scribe as option
                                            QUnit.equal(
                                                widgetInstance.tmplElements.descrSource.val(),
                                                scribeDescription,
                                                'Content in text area for existing description from scribe instance is "' + scribeDescription + '".'
                                            );

                                            // trigger optimize button
                                            widgetInstance.tmplElements.optimizeButton.trigger('click');

                                            window.setTimeout(function() {
                                                replaceDescription = lineParser(widgetInstance.tmplElements.descrTarget[0].value);

                                                // apply new content
                                                widgetInstance.buttons.apply.trigger('click');

                                                window.setTimeout(function() {
                                                    QUnit.equal(
                                                        replaceDescription,
                                                        scribe.content,
                                                        'New content inside the scribe instance is "' + replaceDescription + '".'
                                                    );

                                                    // destroy widget
                                                    aiDescriptionButton.openaiAiContent('destroy');

                                                    window.setTimeout(function() {
                                                        QUnit.ok(
                                                            aiDescriptionButton.data('uiOpenaiAiContent') === undefined,
                                                            'Widget "uiOpenaiAiContent" was destroyed'
                                                        );
                                                        widgetInstance.optimizeDialog && QUnit.ok(
                                                            widgetInstance.optimizeDialog.data('uiUiDialog') === undefined,
                                                            'Widget "optimizeDialog" was destroyed'
                                                        );
                                                        widgetInstance.confirmDialog && QUnit.ok(
                                                            widgetInstance.confirmDialog.data('uiUiDialog') === undefined,
                                                            'Widget "confirmDialog" was destroyed'
                                                        );

                                                        QUnit.start();
                                                    }, 50);
                                                }, 200);
                                            }, 1600);
                                        }, 100);
                                    }, 100);
                                }, 100);
                            }, 100);
                        }, 200);
                    }, 1600);
                }, 100);
            }, 50);
        }, 200);
    });
});